@@ -6,7 +6,6 @@ from __future__ import print_function
 import click
 import sys
 import six  # noqa: F401
-import oci
 
 from services.core.src.oci_cli_virtual_network.generated import virtualnetwork_cli
 
@@ -58,6 +57,9 @@ cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.security_rule_gro
 cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.vcn_group, virtualnetwork_cli.add_vcn_cidr, "add-vcn-cidr")
 cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.vcn_group, virtualnetwork_cli.remove_vcn_cidr, "remove-vcn-cidr")
 cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.vcn_group, virtualnetwork_cli.add_ipv6_vcn_cidr, "add-ipv6-vcn-cidr")
+cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.vcn_group, virtualnetwork_cli.remove_ipv6_vcn_cidr, "remove-ipv6-vcn-cidr")
+cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.subnet_group, virtualnetwork_cli.add_ipv6_subnet_cidr, "add-ipv6-subnet-cidr")
+cli_util.rename_command(virtualnetwork_cli, virtualnetwork_cli.subnet_group, virtualnetwork_cli.remove_ipv6_subnet_cidr, "remove-ipv6-subnet-cidr")
 
 # help for oci network ip-sec-connection create --static-routes
 network_create_ip_sec_connection_static_routes_example = """'["10.0.0.0/16"]'"""
@@ -110,7 +112,7 @@ If you're listing all the private IPs associated with a given subnet or VNIC, th
 cli_util.get_param(virtualnetwork_cli.update_vnic, 'skip_source_dest_check').type = click.BOOL
 
 
-@cli_util.copy_params_from_generated_command(virtualnetwork_cli.create_private_ip, params_to_exclude=[''])
+@cli_util.copy_params_from_generated_command(virtualnetwork_cli.create_private_ip, params_to_exclude=['max_wait_seconds', 'wait_for_state', 'wait_interval_seconds'])
 @virtualnetwork_cli.vnic_group.command(name='assign-private-ip', help="""Assigns a secondary private IP address to the specified VNIC. The secondary private IP must be in the same subnet as the VNIC.
 This command can also be used to move an existing secondary private IP to the specified VNIC.
 
@@ -283,7 +285,7 @@ def unassign_private_ip(ctx, from_json, vnic_id, ip_address):
 @click.pass_context
 @json_skeleton_utils.json_skeleton_generation_handler(input_params_to_complex_types={'defined-tags': {'module': 'core', 'class': 'dict(str, dict(str, object))'}, 'freeform-tags': {'module': 'core', 'class': 'dict(str, string)'}}, output_type={'module': 'core', 'class': 'PrivateIp'})
 @cli_util.wrap_exceptions
-def assign_ipv6(ctx, from_json, vnic_id, defined_tags, display_name, freeform_tags, ip_address, unassign_if_already_assigned):
+def assign_ipv6(ctx, from_json, vnic_id, defined_tags, display_name, freeform_tags, ip_address, unassign_if_already_assigned, ipv6_subnet_cidr):
     networking_client = cli_util.build_client('core', 'virtual_network', ctx)
 
     # First we get the VNIC because we need to know the subnet OCID for the ListIpv6s call
@@ -294,7 +296,7 @@ def assign_ipv6(ctx, from_json, vnic_id, defined_tags, display_name, freeform_ta
     if ip_address is not None:
         # Try and see whether the IP is already in use by calling ListIpv6s with the IP address and subnet. In this case, we don't
         # worry about pagination because we expect at most 1 entry
-        list_ips_response = networking_client.list_ipv6s(ip_address=ip_address, subnet_id=subnet_id, vnic_id=vnic_id)
+        list_ips_response = networking_client.list_ipv6s(ip_address=ip_address, subnet_id=subnet_id)
         list_ips_response_data = list_ips_response.data
 
         if list_ips_response_data is not None:
@@ -334,6 +336,9 @@ def assign_ipv6(ctx, from_json, vnic_id, defined_tags, display_name, freeform_ta
     if freeform_tags is not None:
         assign_ip_request_body['freeformTags'] = cli_util.parse_json_parameter("freeform_tags", freeform_tags)
 
+    if ipv6_subnet_cidr is not None:
+        assign_ip_request_body['ipv6SubnetCidr'] = ipv6_subnet_cidr
+
     # If we are here then either the IP address does not exist or it is a candidate to be moved
     if not is_ip_reassignment:
         if ip_address is not None:
@@ -360,7 +365,7 @@ def assign_ipv6(ctx, from_json, vnic_id, defined_tags, display_name, freeform_ta
 def unassign_ipv6(ctx, from_json, vnic_id, ip_address):
     networking_client = cli_util.build_client('core', 'virtual_network', ctx)
 
-    list_ips_response = networking_client.list_ipv6s(ip_address=ip_address, vnic_id=vnic_id)
+    list_ips_response = networking_client.list_ipv6s(vnic_id=vnic_id)
 
     if list_ips_response.data is None:
         sys.exit('No IP addresses found for VNIC {}'.format(vnic_id))
@@ -615,172 +620,8 @@ def update_network_security_group_security_rules_extended(ctx, **kwargs):
 @click.pass_context
 @json_skeleton_utils.json_skeleton_generation_handler(input_params_to_complex_types={'cross-connect-mappings': {'module': 'core', 'class': 'list[CrossConnectMapping]'}, 'defined-tags': {'module': 'core', 'class': 'dict(str, dict(str, object))'}, 'freeform-tags': {'module': 'core', 'class': 'dict(str, string)'}, 'public-prefixes': {'module': 'core', 'class': 'list[CreateVirtualCircuitPublicPrefixDetails]'}}, output_type={'module': 'core', 'class': 'VirtualCircuit'})
 @cli_util.wrap_exceptions
-def create_virtual_circuit_extended(ctx, from_json, wait_for_state, max_wait_seconds, wait_interval_seconds, compartment_id, type, bandwidth_shape_name, cross_connect_mappings, routing_policy, customer_bgp_asn, customer_asn, defined_tags, display_name, freeform_tags, gateway_id, provider_name, provider_service_id, provider_service_key_name, provider_service_name, public_prefixes):
-    kwargs = {}
-
-    _details = {}
-    _details['compartmentId'] = compartment_id
-    _details['type'] = type
-
-    if bandwidth_shape_name is not None:
-        _details['bandwidthShapeName'] = bandwidth_shape_name
-
-    if cross_connect_mappings is not None:
-        _details['crossConnectMappings'] = cli_util.parse_json_parameter("cross_connect_mappings", cross_connect_mappings)
-
-    if routing_policy is not None:
-        _details['routingPolicy'] = routing_policy
-
-    if customer_bgp_asn is not None:
-        _details['customerBgpAsn'] = customer_bgp_asn
-
-    if customer_asn is not None:
-        _details['customerAsn'] = customer_asn
-
-    if defined_tags is not None:
-        _details['definedTags'] = cli_util.parse_json_parameter("defined_tags", defined_tags)
-
-    if display_name is not None:
-        _details['displayName'] = display_name
-
-    if freeform_tags is not None:
-        _details['freeformTags'] = cli_util.parse_json_parameter("freeform_tags", freeform_tags)
-
-    if gateway_id is not None:
-        _details['gatewayId'] = gateway_id
-
-    if provider_name is not None:
-        _details['providerName'] = provider_name
-
-    if provider_service_id is not None:
-        _details['providerServiceId'] = provider_service_id
-
-    if provider_service_key_name is not None:
-        _details['providerServiceKeyName'] = provider_service_key_name
-
-    if provider_service_name is not None:
-        _details['providerServiceName'] = provider_service_name
-
-    if public_prefixes is not None:
-        _details['publicPrefixes'] = cli_util.parse_json_parameter("public_prefixes", public_prefixes)
-
-    client = cli_util.build_client('core', 'virtual_network', ctx)
-    result = client.create_virtual_circuit(
-        create_virtual_circuit_details=_details,
-        **kwargs
-    )
-    if wait_for_state:
-
-        if hasattr(client, 'get_virtual_circuit') and callable(getattr(client, 'get_virtual_circuit')):
-            try:
-                wait_period_kwargs = {}
-                if max_wait_seconds is not None:
-                    wait_period_kwargs['max_wait_seconds'] = max_wait_seconds
-                if wait_interval_seconds is not None:
-                    wait_period_kwargs['max_interval_seconds'] = wait_interval_seconds
-
-                click.echo('Action completed. Waiting until the resource has entered state: {}'.format(wait_for_state), file=sys.stderr)
-                result = oci.wait_until(client, client.get_virtual_circuit(result.data.id), 'lifecycle_state', wait_for_state, **wait_period_kwargs)
-            except oci.exceptions.MaximumWaitTimeExceeded as e:
-                # If we fail, we should show an error, but we should still provide the information to the customer
-                click.echo('Failed to wait until the resource entered the specified state. Outputting last known resource state', file=sys.stderr)
-                cli_util.render_response(result, ctx)
-                sys.exit(2)
-            except Exception:
-                click.echo('Encountered error while waiting for resource to enter the specified state. Outputting last known resource state', file=sys.stderr)
-                cli_util.render_response(result, ctx)
-                raise
-        else:
-            click.echo('Unable to wait for the resource to enter the specified state', file=sys.stderr)
-    cli_util.render_response(result, ctx)
-
-
-@cli_util.copy_params_from_generated_command(virtualnetwork_cli.update_virtual_circuit, params_to_exclude=[''])
-@virtualnetwork_cli.virtual_circuit_group.command(name=cli_util.override('update_virtual_circuit.command_name', 'update'), help=virtualnetwork_cli.update_virtual_circuit.help)
-@click.pass_context
-@json_skeleton_utils.json_skeleton_generation_handler(input_params_to_complex_types={'cross-connect-mappings': {'module': 'core', 'class': 'list[CrossConnectMapping]'}, 'defined-tags': {'module': 'core', 'class': 'dict(str, dict(str, object))'}, 'freeform-tags': {'module': 'core', 'class': 'dict(str, string)'}}, output_type={'module': 'core', 'class': 'VirtualCircuit'})
-@cli_util.wrap_exceptions
-def update_virtual_circuit_extended(ctx, from_json, force, wait_for_state, max_wait_seconds, wait_interval_seconds, virtual_circuit_id, bandwidth_shape_name, cross_connect_mappings, routing_policy, customer_bgp_asn, customer_asn, defined_tags, display_name, freeform_tags, gateway_id, provider_state, provider_service_key_name, reference_comment, if_match):
-
-    if isinstance(virtual_circuit_id, six.string_types) and len(virtual_circuit_id.strip()) == 0:
-        raise click.UsageError('Parameter --virtual-circuit-id cannot be whitespace or empty string')
-    if not force:
-        if cross_connect_mappings or routing_policy or defined_tags or freeform_tags:
-            if not click.confirm("WARNING: Updates to cross-connect-mappings and routing-policy and defined-tags and freeform-tags will replace any existing values. Are you sure you want to continue?"):
-                ctx.abort()
-
-    kwargs = {}
-    if if_match is not None:
-        kwargs['if_match'] = if_match
-
-    _details = {}
-
-    if bandwidth_shape_name is not None:
-        _details['bandwidthShapeName'] = bandwidth_shape_name
-
-    if cross_connect_mappings is not None:
-        _details['crossConnectMappings'] = cli_util.parse_json_parameter("cross_connect_mappings", cross_connect_mappings)
-
-    if routing_policy is not None:
-        _details['routingPolicy'] = routing_policy
-
-    if customer_bgp_asn is not None:
-        _details['customerBgpAsn'] = customer_bgp_asn
-
-    if customer_asn is not None:
-        _details['customerAsn'] = customer_asn
-
-    if defined_tags is not None:
-        _details['definedTags'] = cli_util.parse_json_parameter("defined_tags", defined_tags)
-
-    if display_name is not None:
-        _details['displayName'] = display_name
-
-    if freeform_tags is not None:
-        _details['freeformTags'] = cli_util.parse_json_parameter("freeform_tags", freeform_tags)
-
-    if gateway_id is not None:
-        _details['gatewayId'] = gateway_id
-
-    if provider_state is not None:
-        _details['providerState'] = provider_state
-
-    if provider_service_key_name is not None:
-        _details['providerServiceKeyName'] = provider_service_key_name
-
-    if reference_comment is not None:
-        _details['referenceComment'] = reference_comment
-
-    client = cli_util.build_client('core', 'virtual_network', ctx)
-    result = client.update_virtual_circuit(
-        virtual_circuit_id=virtual_circuit_id,
-        update_virtual_circuit_details=_details,
-        **kwargs
-    )
-    if wait_for_state:
-
-        if hasattr(client, 'get_virtual_circuit') and callable(getattr(client, 'get_virtual_circuit')):
-            try:
-                wait_period_kwargs = {}
-                if max_wait_seconds is not None:
-                    wait_period_kwargs['max_wait_seconds'] = max_wait_seconds
-                if wait_interval_seconds is not None:
-                    wait_period_kwargs['max_interval_seconds'] = wait_interval_seconds
-
-                click.echo('Action completed. Waiting until the resource has entered state: {}'.format(wait_for_state), file=sys.stderr)
-                result = oci.wait_until(client, client.get_virtual_circuit(result.data.id), 'lifecycle_state', wait_for_state, **wait_period_kwargs)
-            except oci.exceptions.MaximumWaitTimeExceeded as e:
-                # If we fail, we should show an error, but we should still provide the information to the customer
-                click.echo('Failed to wait until the resource entered the specified state. Outputting last known resource state', file=sys.stderr)
-                cli_util.render_response(result, ctx)
-                sys.exit(2)
-            except Exception:
-                click.echo('Encountered error while waiting for resource to enter the specified state. Outputting last known resource state', file=sys.stderr)
-                cli_util.render_response(result, ctx)
-                raise
-        else:
-            click.echo('Unable to wait for the resource to enter the specified state', file=sys.stderr)
-    cli_util.render_response(result, ctx)
+def create_virtual_circuit_extended(ctx, **kwargs):
+    ctx.invoke(virtualnetwork_cli.create_virtual_circuit, **kwargs)
 
 
 # DRG Attachment
