@@ -4,7 +4,15 @@
 
 from services.limits.src.oci_cli_limits.generated import limits_service_cli
 from services.limits.src.oci_cli_quotas.generated import quotas_cli
+from oci_cli import cli_util
+
 
 # remove quotas token from commands
 limits_service_cli.limits_service_group.commands.pop(quotas_cli.quotas_root_group.name)
 limits_service_cli.limits_service_group.add_command(quotas_cli.quota_group)
+
+# oci limits quota add -> oci limits quota addlock
+cli_util.rename_command(quotas_cli, quotas_cli.quota_group, quotas_cli.add_quota_lock, "addlock")
+
+# oci limits quota remove -> oci limits quota removelock
+cli_util.rename_command(quotas_cli, quotas_cli.quota_group, quotas_cli.remove_quota_lock, "removelock")
