@@ -11,13 +11,61 @@ import json
 from oci_cli import cli_util
 from oci_cli import json_skeleton_utils
 from oci_cli import custom_types  # noqa: F401
+from oci_cli.aliasing import CommandGroupWithAlias
 from services.rover.src.constants import ROVER_WORKLOAD_TYPE_IMAGE
 from services.rover.src.oci_cli_rover.generated import rover_service_cli
-from services.rover.src.oci_cli_rover.rover_utils import get_compute_image_helper, export_compute_image_helper, \
-    prompt_for_secrets, prompt_for_workload_delete, export_compute_image_status_helper, modify_image_workload_name
+from services.rover.src.oci_cli_rover.rover_utils import export_compute_image_helper, \
+    prompt_for_secrets, prompt_for_workload_delete, export_compute_image_status_helper, \
+    validate_get_image, prepare_image_workload_data, prepare_bucket_workload_data, \
+    create_master_key_policy_rover_resource, remove_additional_params_after_policy
+from services.rover.src.oci_cli_rover_bundle.generated import roverbundle_cli
 from services.rover.src.oci_cli_rover_node.generated import rovernode_cli
 from oci.util import formatted_flat_dict
 
+
+@click.command('ca-bundle', cls=CommandGroupWithAlias, help="""Rover Node Certificate Bundle.""")
+@cli_util.help_option_group
+def rover_ca_bundle_group():
+    pass
+
+
+@click.command('certificate', cls=CommandGroupWithAlias, help="""Rover Node Certificate.""")
+@cli_util.help_option_group
+def rover_certificate_group():
+    pass
+
+
+@click.command('certificate-authority', cls=CommandGroupWithAlias, help="""Rover Node Certificate Authority.""")
+@cli_util.help_option_group
+def rover_certificate_authority_group():
+    pass
+
+
+@click.command('rover-bundle-request', cls=CommandGroupWithAlias, help="""Rover Bundle Requests for Node.""")
+@cli_util.help_option_group
+def rover_bundle_request_group():
+    pass
+
+
+@click.command('rover-bundle', cls=CommandGroupWithAlias, help="""Rover Bundle for Node.""")
+@cli_util.help_option_group
+def rover_bundle_group():
+    pass
+
+
+@click.command('rover-bundle-version', cls=CommandGroupWithAlias, help="""Rover Bundle Version for Cluster.""")
+@cli_util.help_option_group
+def rover_bundle_version_group():
+    pass
+
+
+rovernode_cli.rover_node_group.add_command(rover_ca_bundle_group)
+rovernode_cli.rover_node_group.add_command(rover_certificate_group)
+rovernode_cli.rover_node_group.add_command(rover_certificate_authority_group)
+rovernode_cli.rover_node_group.add_command(rover_bundle_request_group)
+rovernode_cli.rover_node_group.add_command(rover_bundle_group)
+rovernode_cli.rover_node_group.add_command(rover_bundle_version_group)
+
 cli_util.rename_command(rover_service_cli, rover_service_cli.rover_service_group, rovernode_cli.rover_node_group, 'node')
 cli_util.rename_command(rover_service_cli, rovernode_cli.rover_node_group, rovernode_cli.get_rover_node, 'show')
 cli_util.rename_command(rover_service_cli, rovernode_cli.rover_node_group, rovernode_cli.list_rover_nodes, 'list')
@@ -25,6 +73,52 @@ cli_util.rename_command(rover_service_cli, rovernode_cli.rover_node_group, rover
 cli_util.rename_command(rover_service_cli, rovernode_cli.rover_node_group, rovernode_cli.get_rover_node_certificate, 'get-certificate')
 
 
+# oci rover node rover-node-action-retrieve-ca-bundle -> oci rover node ca-bundle get
+rovernode_cli.rover_node_group.commands.pop(rovernode_cli.rover_node_action_retrieve_ca_bundle.name)
+rover_ca_bundle_group.add_command(rovernode_cli.rover_node_action_retrieve_ca_bundle)
+cli_util.rename_command(rovernode_cli, rover_ca_bundle_group, rovernode_cli.rover_node_action_retrieve_ca_bundle, "get")
+
+# # oci rover node rover-node-generate-certificate -> oci rover node certificate create
+rovernode_cli.rover_node_group.commands.pop(rovernode_cli.rover_node_generate_certificate.name)
+rover_certificate_group.add_command(rovernode_cli.rover_node_generate_certificate)
+cli_util.rename_command(rovernode_cli, rover_certificate_group, rovernode_cli.rover_node_generate_certificate, "create")
+
+# oci rover node rover-node-renew-certificate -> oci rover node certificate update
+rovernode_cli.rover_node_group.commands.pop(rovernode_cli.rover_node_renew_certificate.name)
+rover_certificate_group.add_command(rovernode_cli.rover_node_renew_certificate)
+cli_util.rename_command(rovernode_cli, rover_certificate_group, rovernode_cli.rover_node_renew_certificate, "update")
+
+# oci rover node rover-node-retrieve-leaf-certificate -> oci rover node certificate get-leaf-certificate
+rovernode_cli.rover_node_group.commands.pop(rovernode_cli.rover_node_retrieve_leaf_certificate.name)
+rover_certificate_group.add_command(rovernode_cli.rover_node_retrieve_leaf_certificate)
+cli_util.rename_command(rovernode_cli, rover_certificate_group, rovernode_cli.rover_node_retrieve_leaf_certificate, "get-leaf-certificate")
+
+# oci rover node rover-node-replace-certificate-authority -> oci rover node certificate-authority update-root-ca
+rovernode_cli.rover_node_group.commands.pop(rovernode_cli.rover_node_replace_certificate_authority.name)
+rover_certificate_authority_group.add_command(rovernode_cli.rover_node_replace_certificate_authority)
+cli_util.rename_command(rovernode_cli, rover_certificate_authority_group, rovernode_cli.rover_node_replace_certificate_authority, "update-root-ca")
+
+# oci rover rover-bundle rover-node list-rover-node-rover-bundle-requests -> oci rover node rover-bundle-request list
+roverbundle_cli.rover_node_group.commands.pop(roverbundle_cli.list_rover_node_rover_bundle_requests.name)
+rover_bundle_request_group.add_command(roverbundle_cli.list_rover_node_rover_bundle_requests)
+cli_util.rename_command(roverbundle_cli, rover_bundle_request_group, roverbundle_cli.list_rover_node_rover_bundle_requests, "list")
+
+#  oci rover rover-bundle rover-node request-bundle -> oci rover node rover-bundle copy-to-customer
+roverbundle_cli.rover_node_group.commands.pop(roverbundle_cli.request_bundle_rover_node.name)
+rover_bundle_group.add_command(roverbundle_cli.request_bundle_rover_node)
+cli_util.rename_command(roverbundle_cli, rover_bundle_group, roverbundle_cli.request_bundle_rover_node, "copy-to-customer")
+
+#  oci rover rover-bundle rover-node retrieve-bundle-status -> oci rover node rover-bundle get-status
+roverbundle_cli.rover_node_group.commands.pop(roverbundle_cli.retrieve_bundle_status_rover_node.name)
+rover_bundle_group.add_command(roverbundle_cli.retrieve_bundle_status_rover_node)
+cli_util.rename_command(roverbundle_cli, rover_bundle_group, roverbundle_cli.retrieve_bundle_status_rover_node, "get-status")
+
+#  oci rover rover-bundle rover-node retrieve-available-bundle-versions -> oci rover node rover-bundle-version get
+roverbundle_cli.rover_node_group.commands.pop(roverbundle_cli.retrieve_available_bundle_versions_rover_node.name)
+rover_bundle_version_group.add_command(roverbundle_cli.retrieve_available_bundle_versions_rover_node)
+cli_util.rename_command(roverbundle_cli, rover_bundle_version_group, roverbundle_cli.retrieve_available_bundle_versions_rover_node, "get")
+
+
 def complex_shipping_address_param(**kwargs):
 
     kwargs['customer_shipping_address'] = {}
@@ -129,8 +223,10 @@ def setup_identity_helper(ctx, **kwargs):
     return create_policies_result
 
 
-@cli_util.copy_params_from_generated_command(rovernode_cli.create_rover_node, params_to_exclude=['customer_shipping_address', 'node_workloads', 'public_key', 'serial_number', 'super_user_password', 'unlock_passphrase', 'oracle_shipping_tracking_url', 'shipping_vendor', 'time_pickup_expected'])
+@cli_util.copy_params_from_generated_command(rovernode_cli.create_rover_node, params_to_exclude=['customer_shipping_address', 'data_validation_code', 'import_compartment_id', 'import_file_bucket', 'is_import_requested', 'node_workloads', 'public_key', 'serial_number', 'super_user_password', 'unlock_passphrase', 'oracle_shipping_tracking_url', 'shipping_vendor', 'time_pickup_expected', 'certificate_authority_id'])
 @rovernode_cli.rover_node_group.command(name=rovernode_cli.create_rover_node.name, help=rovernode_cli.create_rover_node.help)
+@cli_util.option('--issuer-certificate-authority-id', help="""The certificateAuthorityId of subordinate/intermediate certificate authority.""")
+@cli_util.option('--shape', required=True, help=u"""Shape of the node on Rover device""")
 @cli_util.option('--addressee', help=u"""Company or person to send the appliance to""")
 @cli_util.option('--care-of', help=u"""Place/person to direct the package to.""")
 @cli_util.option('--address1', help=u"""Address line 1.""")
@@ -144,6 +240,8 @@ def setup_identity_helper(ctx, **kwargs):
 @cli_util.option('--phone-number', help=u"""Phone number.""")
 @cli_util.option('--email', help=u"""Email address.""")
 @cli_util.option('--setup-identity', help=u"""Creating dynamic group and Assigning Policies for Rover node""", is_flag=True)
+@cli_util.option('--policy-compartment-id', help=u"""Compartment ID where the master key policy (if master key provided) would be created""")
+@cli_util.option('--policy-name', help=u"""Display name for the policy to be created for the master key (if provided)""")
 @click.pass_context
 @json_skeleton_utils.json_skeleton_generation_handler(input_params_to_complex_types={'customer-shipping-address': {'module': 'rover', 'class': 'ShippingAddress'}, 'node-workloads': {'module': 'rover', 'class': 'list[object]'}, 'freeform-tags': {'module': 'rover', 'class': 'dict(str, string)'}, 'defined-tags': {'module': 'rover', 'class': 'dict(str, dict(str, object))'}, 'system-tags': {'module': 'core', 'class': 'dict(str, dict(str, object))'}}, output_type={'module': 'rover', 'class': 'RoverNode'})
 @cli_util.wrap_exceptions
@@ -153,6 +251,7 @@ def create_rover_node_extended(ctx, **kwargs):
     _details = {}
     _details['displayName'] = kwargs['display_name']
     _details['compartmentId'] = kwargs['compartment_id']
+    _details['shape'] = kwargs['shape']
 
     if kwargs['customer_shipping_address'] is not None:
         _details['customerShippingAddress'] = cli_util.parse_json_parameter("customer_shipping_address", kwargs['customer_shipping_address'])
@@ -193,6 +292,34 @@ def create_rover_node_extended(ctx, **kwargs):
     if kwargs['system_tags'] is not None:
         _details['systemTags'] = cli_util.parse_json_parameter("system_tags", kwargs['system_tags'])
 
+    if 'issuer_certificate_authority_id' in kwargs:
+        _details['certificateAuthorityId'] = kwargs['issuer_certificate_authority_id']
+        kwargs.pop('issuer_certificate_authority_id')
+
+    if 'cert_compartment_id' in kwargs:
+        _details['certCompartmentId'] = kwargs['cert_compartment_id']
+
+    if 'cert_key_algorithm' in kwargs:
+        _details['certKeyAlgorithm'] = kwargs['cert_key_algorithm']
+
+    if 'cert_signature_algorithm' in kwargs:
+        _details['certSignatureAlgorithm'] = kwargs['cert_signature_algorithm']
+
+    if 'common_name' in kwargs:
+        _details['commonName'] = kwargs['common_name']
+
+    if 'time_cert_validity_end' in kwargs:
+        _details['timeCertValidityEnd'] = kwargs['time_cert_validity_end']
+
+    # set up policy for master key if provided
+    if kwargs['master_key_id']:
+        create_master_key_policy_rover_resource("node", ctx, **kwargs)
+        _details['masterKeyId'] = kwargs['master_key_id']
+    elif kwargs['policy_name'] or kwargs['policy_compartment_id']:
+        raise click.UsageError('policy-compartment-id or policy-name cannot be provided without master-key-id')
+    # Remove additional parameters of policy from kwargs
+    kwargs = remove_additional_params_after_policy(**kwargs)
+
     # creating rover node
 
     rover_node_kwrags = {}
@@ -258,7 +385,7 @@ def show_rover_node_extended(ctx, **kwargs):
     cli_util.render_response(result, ctx)
 
 
-@cli_util.copy_params_from_generated_command(rovernode_cli.update_rover_node, params_to_exclude=['customer_shipping_address', 'rover_node_id', 'node_workloads', 'public_key', 'serial_number', 'super_user_password', 'unlock_passphrase', 'oracle_shipping_tracking_url', 'shipping_vendor', 'time_pickup_expected'])
+@cli_util.copy_params_from_generated_command(rovernode_cli.update_rover_node, params_to_exclude=['customer_shipping_address', 'data_validation_code', 'import_compartment_id', 'import_file_bucket', 'is_import_requested', 'rover_node_id', 'node_workloads', 'public_key', 'serial_number', 'super_user_password', 'unlock_passphrase', 'oracle_shipping_tracking_url', 'shipping_vendor', 'time_pickup_expected'])
 @rovernode_cli.rover_node_group.command(name=rovernode_cli.update_rover_node.name, help=rovernode_cli.update_rover_node.help)
 @cli_util.option('--node-id', required=True, help=u"""Unique RoverNode identifier""")
 @cli_util.option('--addressee', help=u"""Company or person to send the appliance to""")
@@ -278,10 +405,13 @@ def show_rover_node_extended(ctx, **kwargs):
 @cli_util.wrap_exceptions
 def update_rover_node_extended(ctx, **kwargs):
 
-    client = cli_util.build_client('rover', 'rover_node', ctx)
+    cli_util.build_client('rover', 'rover_node', ctx)
 
     kwargs = complex_shipping_address_param(**kwargs)
-    kwargs['lifecycle_state_details'] = "PENDING_SUBMISSION"
+    if not kwargs['lifecycle_state_details']:
+        pending_lifecycle_state = "PENDING_SUBMISSION"
+        click.echo("WARNING:Rover node will be updated with lifecycle state details " + pending_lifecycle_state)
+        kwargs['lifecycle_state_details'] = pending_lifecycle_state
     kwargs.update({
         'rover_node_id': kwargs['node_id']
     })
@@ -328,10 +458,8 @@ def delete_rover_node_extended(ctx, **kwargs):
 
 @rovernode_cli.rover_node_group.command(name="add-workload", help=u"""Add workload information to Rover Node""")
 @cli_util.option('--node-id', required=True, help=u"""Unique RoverNode identifier""")
-@cli_util.option('--compartment-id', required=True, help=u"""Compartment Id of Bucket""")
 @cli_util.option('--type', required=True, type=custom_types.CliCaseInsensitiveChoice(["BUCKET", "IMAGE"]), help=u"""Type of workload""")
 @cli_util.option('--image-id', help=u"""Object Store Image OCID for the workload""")
-@cli_util.option('--bucket-id', help=u"""Object Store Bucket OCID for the workload""")
 @cli_util.option('--bucket-name', help=u"""Object Store Bucket name for the workload""")
 @cli_util.option('--prefix', help=u"""List of objects with names matching this prefix would be part of this export job.""")
 @cli_util.option('--range-start', help=u"""Object names returned by a list query must be greater or equal to this parameter.""")
@@ -344,39 +472,27 @@ def delete_rover_node_extended(ctx, **kwargs):
 @cli_util.wrap_exceptions
 def add_workload(ctx, **kwargs):
 
-    workload_data = image_id = workload_id = compute_obj = destination_uri = None
+    workload_data = image_id = workload_id = destination_uri = None
     result = get_rover_node_helper(ctx, kwargs['node_id'])
 
     if kwargs['type'].lower() == "bucket":
-        if not ('bucket_id' in kwargs and kwargs['bucket_id']) or not ('bucket_name' in kwargs and kwargs['bucket_name']):
-            raise click.UsageError('Parameter bucket-id and bucket-name cannot be whitespace or empty string')
-        workload_id = kwargs['bucket_id']
-        workload_data = [{
-            "workloadType": "BUCKET", "id": kwargs['bucket_id'], "name": kwargs['bucket_name'], "compartmentId": kwargs['compartment_id'],
-            'prefix': kwargs['prefix'], 'rangeStart': kwargs['range_start'], 'rangeEnd': kwargs['range_end']
-        }]
+        if not ('bucket_name' in kwargs and kwargs['bucket_name']):
+            raise click.UsageError('Parameter bucket-name cannot be whitespace or empty string')
+        workload_id = kwargs['bucket_name']
+        workload_data = prepare_bucket_workload_data(result.data.compartment_id, **kwargs)
 
     elif kwargs['type'].lower() == "image":
-        if 'image_id' in kwargs and not kwargs['image_id']:
-            raise click.UsageError('Parameter image-id cannot be whitespace or empty string')
+        compute_image_obj = validate_get_image(ctx, **kwargs)
         workload_id = image_id = kwargs['image_id']
-        compute_image_obj = get_compute_image_helper(ctx, image_id)
-        compute_image_obj_name = modify_image_workload_name(compute_image_obj.data.display_name)
-        destination_uri = result.data.image_export_par + compute_image_obj_name + "_" + image_id + ".oci"
-        workload_data = [
-            {'workloadType': "IMAGE", 'id': image_id, 'name': compute_image_obj_name,
-             'size': compute_image_obj.data.size_in_mbs, 'compartmentId': kwargs['compartment_id'],
-             }
-        ]
+        workload_data = prepare_image_workload_data(compute_image_obj, image_id)
+        destination_uri = result.data.image_export_par + workload_data[0]['name'] + "_" + image_id + ".oci"
     confirm_prompt = "Would you like to submit the following workload information ? " + formatted_flat_dict(workload_data)
-    if result.data.node_workloads:
-        if any(existing_workload.id == workload_id for existing_workload in result.data.node_workloads):
-            raise click.UsageError("Workload with {} is already attached".format(workload_id))
+    if result.data.node_workloads and len(result.data.node_workloads) > 0 and any(existing_workload.id == workload_id for existing_workload in result.data.node_workloads):
+        raise click.UsageError("Workload with {} is already attached".format(workload_id))
 
-    if not kwargs['force']:
-        if not click.confirm(click.style(confirm_prompt, fg="yellow")):
-            click.echo("Aborting workload selection for Rover Cluster")
-            ctx.abort()
+    if not kwargs['force'] and not click.confirm(click.style(confirm_prompt, fg="yellow")):
+        click.echo("Aborting workload selection for Rover Cluster")
+        ctx.abort()
 
     if kwargs['type'].lower() == "image":
         export_return_response = export_compute_image_helper(ctx, image_id, destination_uri)
@@ -453,7 +569,6 @@ def list_workload(ctx, **kwargs):
 @cli_util.copy_params_from_generated_command(rovernode_cli.change_rover_node_compartment, params_to_exclude=['rover_node_id'])
 @rovernode_cli.rover_node_group.command(name=rovernode_cli.change_rover_node_compartment.name, help=rovernode_cli.change_rover_node_compartment.help)
 @cli_util.option('--node-id', required=True, help=u"""Unique RoverNode identifier""")
-@cli_util.option('--compartment-id', required=True, help=u"""The [OCID]  of the compartment into which the resources should be moved.""")
 @cli_util.option('--if-match', help=u"""For optimistic concurrency control. In the PUT or DELETE call for a resource, set the `if-match` parameter to the value of the etag from a previous GET or POST response for that resource. The resource will be updated or deleted only if the etag you provide matches the resource's current etag value.""")
 @click.pass_context
 @json_skeleton_utils.json_skeleton_generation_handler(input_params_to_complex_types={})
