@@ -16,15 +16,18 @@ from oci.util import Sentinel   # noqa: E402
 import six  # noqa: E402
 import importlib    # noqa: E402
 import re   # noqa: E402
-from .version import __version__    # noqa: E402
-from .aliasing import parameter_alias, CommandGroupWithAlias    # noqa: E402
-from . import help_text_producer    # noqa: E402
-from . import cli_util  # noqa: E402
+from oci_cli.version import __version__    # noqa: E402
+from oci_cli.aliasing import parameter_alias, CommandGroupWithAlias    # noqa: E402
+from oci_cli import help_text_producer    # noqa: E402
+from oci_cli import cli_util  # noqa: E402
 
-from . import cli_constants     # noqa: E402
+from oci_cli import cli_constants     # noqa: E402
+from alloy import alloy_util   # noqa: E402
 from collections import OrderedDict     # noqa: E402
 from oci._vendor import requests    # noqa: E402
-from . import cli_metrics    # noqa: E402
+from oci_cli import cli_metrics    # noqa: E402
+from interactive import cli_interactive    # noqa: E402
+from oci_cli.service_mapping import service_mapping    # noqa: E402
 
 # Enable WARN logging to surface important warnings attached to loading
 # defaults, automatic coercion, or fallback values/endpoints that may impact
@@ -42,6 +45,107 @@ logging.basicConfig(level=logging.WARN)
 
 OCI_CLI_AUTH_CHOICES = [cli_constants.OCI_CLI_AUTH_API_KEY, cli_constants.OCI_CLI_AUTH_INSTANCE_PRINCIPAL, cli_constants.OCI_CLI_AUTH_SESSION_TOKEN, cli_constants.OCI_CLI_AUTH_INSTANCE_OBO_USER, cli_constants.OCI_CLI_AUTH_RESOURCE_PRINCIPAL]
 
+OCI_HELP = 'Oracle Cloud Infrastructure command line interface'
+
+OCI_USER_HELP = f'''Oracle Cloud Infrastructure command line interface, with support for Audit, Block Volume,
+Compute, Database, IAM, Load Balancing, Networking, DNS, File Storage, Email Delivery and Object Storage Services.
+
+Most commands must specify a service, followed by a resource type and then an action. For example, to list users (where
+$T contains the OCID of the current tenant):
+
+  oci iam user list --compartment-id $T
+
+Output is in JSON format.
+
+For information on configuration, see {cli_constants.OCI_CONFIG_DOCUMENTATION}.
+
+Enable our interactive features to guide you through command usage:
+
+  oci -i
+
+For information on interactive features, see {cli_constants.INTERACTIVE_CLI_DOCUMENTATION}.'''
+
+OCI_ALLOY_USER_HELP = '''Oracle Cloud Infrastructure command line interface, with support for subscribed services.
+
+Most commands requires specification of a service, a resource type, and an action.
+While built-in commands don't require those information, it may have other required parameters. For example, to send
+a raw request (where $method is an HTTP method and $uri is a service endpoint):
+
+  oci raw-request --http-method $method --target-uri $uri
+
+Output is in JSON format.
+
+For information on configuration, see https://docs.cloud.oracle.com/Content/API/Concepts/sdkconfig.htm.'''
+
+REALM_SPECIFIC_ENDPOINT_HELP = """Use a realm-specific endpoint, instead of the default region-wide endpoint.
+
+Example::
+    oci os ns get --realm-specific-endpoint
+
+With this flag, your request will be made to a realm-specific service endpoint, such as, "https://mynamespace.objectstorage.us-phoenix-1.oci.customer-oci.com". Use the --debug option to identify which endpoint is used for your request.
+
+This flag has priority over the environment variable OCI_REALM_SPECIFIC_SERVICE_ENDPOINT_TEMPLATE_ENABLED.
+
+To use this flag by default, set this flag to true in your OCI CLI-specific configuration file. For more information, see https://docs.oracle.com/en-us/iaas/Content/API/SDKDocs/cliconfigure.htm
+
+If the flag is used for a service that does not support such feature, the default region-wide endpoint will be used.
+"""
+
+GENERATE_PARAM_JSON_HELP = """Complex input, such as arrays and objects, are passed in JSON format.
+
+When passed the name of an option which takes complex input, this will print out example JSON of what needs to be passed to that option.
+Example::
+    oci compute instance launch --generate_param_json_input_r agent-config > example.json
+
+    cat example.json
+Output::
+
+    {
+      "areAllPluginsDisabled": true,
+      "isManagementDisabled": true,
+      "isMonitoringDisabled": true,
+      "pluginsConfig": [
+        {
+          "desiredState": "string",
+          "name": "string"
+        },
+        {
+          "desiredState": "string",
+          "name": "string"
+        }
+      ]
+    }
+
+Edit the example.json file with correct values and use it as below ::
+
+    oci compute instance launch --agent-config file://example.json
+
+"""
+GENERATE_FULL_COMMAND_JSON_HELP = """Prints out a JSON document which represents all possible options that can be provided to this command.
+
+This JSON document can be saved to a file, modified with the appropriate option values, and then passed back via the --from-json option. This provides an alternative to typing options out on the command line.
+Example::
+    oci os bucket get --generate_full_command_json_input_r  > example.json
+    cat example.json
+Output::
+
+    {
+      "bucketName": "string",
+      "fields": [
+        "approximateCount|approximateSize|autoTiering"
+      ],
+      "ifMatch": "string",
+      "ifNoneMatch": "string",
+      "name": "string",
+      "namespace": "string",
+      "namespaceName": "string"
+    }
+
+Edit the example.json file with correct values and use it as below ::
+
+    oci os bucket get --from-json file://example.json
+"""
+
 
 def eager_load_cli_rc_file(ctx, param, value):
     expanded_rc_default_location = os.path.expandvars(os.path.expanduser(cli_constants.CLI_RC_DEFAULT_LOCATION))
@@ -236,11 +340,11 @@ def find_latest_release_info(ctx, param, value):
                 if key == version:
                     if version_up_to_date:
                         click.echo(click.style("Version {} is up to date with latest release".format(version)))
-                    ctx.exit()
+                    sys.exit()
                 version_up_to_date = False
                 click.echo(key)
                 click.echo(od[key])
-    ctx.exit()
+    sys.exit()
 
 
 def find_latest_release_version(ctx, param, value):
@@ -273,30 +377,19 @@ def find_latest_release_version(ctx, param, value):
         current_version_list = [int(x) for x in current_version.split(".")]
         latest_version_list = [int(x) for x in latest_version.split(".")]
         if current_version_list < latest_version_list:
-            click.echo(click.style("You are using OCI CLI version {}, however version {} is available. You should consider upgrading using"
-                                   " https://docs.cloud.oracle.com/en-us/iaas/Content/API/SDKDocs/cliupgrading.htm".format(current_version, latest_version), fg='red'))
+            click.echo(click.style("You are using OCI CLI version {}, however version {} is available. You should consider upgrading using {}".format(
+                current_version, latest_version, cli_constants.UPGRADING_CLI_DOCUMENTATION), fg='red'))
             exit_code = 1
 
-    ctx.exit(exit_code)
+    sys.exit(exit_code)
 
 
 @click.command(name='oci', cls=CommandGroupWithAlias, invoke_without_command=True,
-               context_settings=dict(allow_interspersed_args=True, ignore_unknown_options=True),
-               help="""Oracle Cloud Infrastructure command line interface, with support for Audit, Block Volume,
-Compute, Database, IAM, Load Balancing, Networking, DNS, File Storage, Email Delivery and Object Storage Services.
-
-Most commands must specify a service, followed by a resource type and then an action. For example, to list users (where
-$T contains the OCID of the current tenant):
-
-  oci iam user list --compartment-id $T
-
-Output is in JSON format.
-
-For information on configuration, see https://docs.cloud.oracle.com/Content/API/Concepts/sdkconfig.htm.""")
+               context_settings=dict(allow_interspersed_args=True, ignore_unknown_options=True), help=OCI_HELP)
 @click.version_option(__version__, '-v', '--version', message='%(version)s')
 @click.option('--release-info', is_flag=True, show_default=False, callback=find_latest_release_info,
               expose_value=False, is_eager=True, help='Prints ChangeLog difference between current installed version and '
-                                                      'latest released version. Please visit https://raw.githubusercontent.com/oracle/oci-cli/master/CHANGELOG.rst for more info')
+                                                      f'latest released version. Please visit {cli_constants.GITHUB_CHANGELOG} for more info')
 @click.option('--latest-version', is_flag=True, show_default=False, callback=find_latest_release_version,
               expose_value=False, is_eager=True, help='Prints latest released version.')
 @click.option('--config-file',
@@ -308,38 +401,51 @@ For information on configuration, see https://docs.cloud.oracle.com/Content/API/
 @click.option('--cli-rc-file', '--defaults-file',
               default=cli_constants.CLI_RC_DEFAULT_LOCATION, show_default=True,
               is_eager=True, callback=eager_load_cli_rc_file,
-              help='The path to the OCI CLI-specific configuration file, containing parameter default values and other configuration information such as command aliases and predefined queries. The --defaults-file option is deprecated and you should use the --cli-rc-file option instead.')
+              help=f'''The path to the OCI CLI-specific configuration file, containing parameter default values and other configuration information such as command aliases and predefined queries.
+
+The --defaults-file option is deprecated and you should use the --cli-rc-file option instead.
+
+For more information about the cli configuration file, see {cli_constants.CLI_CONFIG_DOCUMENTATION}.''')
 @click.option('--opc-request-id', '--opc-client-request-id', '--request-id', 'request_id',
               help='The request id to use for tracking the request.')
 @click.option('--region', callback=read_values_from_env, help='The region to make calls against.  For a list of valid region names use the command: "oci iam region list".')
 @click.option('--endpoint', callback=read_values_from_env, help='The value to use as the service endpoint, including any required API version path. For example: "https://iaas.us-phoenix-1.oracle.com/20160918". This will override the default service endpoint / API version path. Note: The --region parameter is the recommended way of targeting different regions.')
+@click.option('--realm-specific-endpoint', is_flag=True, help=REALM_SPECIFIC_ENDPOINT_HELP)
+@click.option('--connection-timeout', 'connection_timeout', type=click.INT, callback=read_values_from_env, help='The value of the connection timeout in seconds to make establish connection from sdk to services. This will override the default connection timeout value of 10 secs. ')
+@click.option('--read-timeout', 'read_timeout', type=click.INT, callback=read_values_from_env, help='The value of the read timeout in seconds to wait for service calls to send response to sdk. This will override the default read timeout value of 60 secs. ')
 @click.option('--cert-bundle', callback=read_values_from_env, help='The full path to a CA certificate bundle to be used for SSL verification. This will override the default CA certificate bundle.')
 @click.option('--output', type=click.Choice(choices=['json', 'table']), help='The output format. [Default is json]')
-@click.option('--query', help="""JMESPath query [http://jmespath.org/] to run on the response JSON before output.
+@click.option('--query', help=f"""JMESPath query [http://jmespath.org/] to run on the response JSON before output.
 
 Queries can be entered directly on the command line or referenced from the [OCI_CLI_COMMAND_ALIASES] section of your configuration file by using the syntax query://<query_name>, for example query://get_id_and_name
+
+For more information, see the Using Queries section at {cli_constants.INPUT_OUTPUT_DOCUMENTATION}
 """)
 @click.option('--raw-output', is_flag=True, help='If the output of a given query is a single string value, this will return the string without surrounding quotes')
 @click.option('--auth', type=click.Choice(choices=OCI_CLI_AUTH_CHOICES), help='The type of auth to use for the API request. By default the API key in your config file will be used.  This value can also be provided in the {auth_env_var} environment variable.'.format(auth_env_var=cli_constants.OCI_CLI_AUTH_ENV_VAR))
 @click.option('--auth-purpose', help='The The auth purpose which can be used in conjunction with --auth.')
-@click.option('--generate-full-command-json-input', is_flag=True, is_eager=True, help="""Prints out a JSON document which represents all possible options that can be provided to this command.
-
-This JSON document can be saved to a file, modified with the appropriate option values, and then passed back via the --from-json option. This provides an alternative to typing options out on the command line.""")
-@click.option('--generate-param-json-input', is_eager=True, help="""Complex input, such as arrays and objects, are passed in JSON format.
-
-When passed the name of an option which takes complex input, this will print out example JSON of what needs to be passed to that option.""")
+@click.option('--generate-full-command-json-input', is_flag=True, is_eager=True, help=GENERATE_FULL_COMMAND_JSON_HELP)
+@click.option('--generate-param-json-input', is_eager=True, help=GENERATE_PARAM_JSON_HELP)
 @click.option('--no-retry', is_flag=True, help='Disable retry logic for calls to services.')
+@click.option('--max-retries', type=click.INT, help='Maximum number of retry calls to be made to the service. For most commands, 5 attempts will be made. For operations with binary bodies, retries are disabled')
+@click.option('--proxy', help="""If the environment the CLI runs in requires use of a proxy server for HTTP requests, the details of the proxy server to use. Examples:
+
+http://proxy.example.org:3128/ (uses proxy.example.org on port 3128)
+\b
+http://user:pass@proxy.example.org:3128/ (if your proxy requires a username and password to authenticate)""")
 @click.option('-d', '--debug', is_flag=True, help='Show additional debug information.')
+@click.option('-i', '--cli-auto-prompt', is_flag=True, help=f'''Use the interactive features for autocompletion and quick view of command reference.
+
+For information on interactive features, see {cli_constants.INTERACTIVE_CLI_DOCUMENTATION}.''')
 @click.option('-?', '-h', '--help', is_flag=True, help='For detailed help on the individual OCI CLI command, enter <command> --help.')
 @click.pass_context
-def cli(ctx, config_file, profile, defaults_file, request_id, region, endpoint, cert_bundle, output, query, raw_output, auth, auth_purpose, no_retry, generate_full_command_json_input, generate_param_json_input, debug, help):
-    # Show help in any case if there are no subcommands, or if the help option
-    # is used but there are subcommands, then set a flag for user later.
-    if not ctx.invoked_subcommand:
-        click.echo(ctx.get_help(), color=ctx.color)
-        ctx.exit()
+def cli(ctx, config_file, profile, cli_rc_file, request_id, region, endpoint, realm_specific_endpoint, cert_bundle, output, query, raw_output, auth, auth_purpose, no_retry, max_retries, generate_full_command_json_input, generate_param_json_input, proxy, debug, cli_auto_prompt, connection_timeout, read_timeout, help):
 
-    if profile == Sentinel(DEFAULT_PROFILE):
+    click.exceptions.UsageError.show = cli_util.update_click_help_message
+    if max_retries and no_retry:
+        raise click.UsageError('The option --max-retries is not applicable when using the --no-retry flag.')
+
+    if str(profile) == str(Sentinel(DEFAULT_PROFILE)):
         # if --profile is not supplied, fallback accordingly:
         #   - if OCI_CLI_PROFILE exists, use that
         #   - if default_profile is specified in oci_cli_rc then use that
@@ -365,20 +471,25 @@ def cli(ctx, config_file, profile, defaults_file, request_id, region, endpoint,
     initial_dict = {
         'config_file': config_file,
         'profile': profile,
-        'defaults_file': defaults_file,
+        'cli_rc_file': cli_rc_file,
         'request_id': request_id,
         'region': region,
         'endpoint': endpoint,
+        'realm_specific_endpoint': realm_specific_endpoint,
+        'connection_timeout': connection_timeout,
+        'read_timeout': read_timeout,
         'cert_bundle': cert_bundle,
         'output': output,
         'query': query,
         'raw_output': raw_output,
         'generate_full_command_json_input': generate_full_command_json_input,
         'generate_param_json_input': generate_param_json_input,
+        'proxy': proxy,
         'debug': debug,
         'no_retry': no_retry,
         'auth': auth,
-        'auth_purpose': auth_purpose
+        'auth_purpose': auth_purpose,
+        'max_attempts': max_retries
     }
 
     if not ctx.obj:
@@ -386,11 +497,20 @@ def cli(ctx, config_file, profile, defaults_file, request_id, region, endpoint,
     else:
         ctx.obj.update(initial_dict)
 
-    load_default_values(ctx, defaults_file, profile)
+    load_default_values(ctx, cli_rc_file, profile)
+
+    # Show help in any case if there are no subcommands, or if the help option
+    # is used but there are subcommands, then set a flag for user later.
+    if not ctx.invoked_subcommand and not (cli_constants.OCI_CLI_AUTO_PROMPT_ENV_VAR in os.environ or cli_auto_prompt):
+        echo_help(ctx)
+        ctx.exit()
 
     if help:
         ctx.obj['help'] = True
         if is_top_level_help(ctx) and not cli_util.parse_boolean(ctx.obj.get('settings', {}).get(cli_constants.CLI_RC_GENERIC_SETTINGS_USE_CLICK_HELP, False)):
+            # check if customer is trying to access a service that they have not subscribed
+            if not cli_util.is_service_accessible(ctx, ctx.invoked_subcommand):
+                ctx.exit()
             help_text_producer.render_help_text(ctx, [sys.argv[1]])
 
     cli_metrics.Metrics.update_metric("NUM_INVOCATIONS", ctx.obj['debug'])
@@ -408,6 +528,31 @@ def cli(ctx, config_file, profile, defaults_file, request_id, region, endpoint,
     except Exception:
         pass
 
+    # Show auto prompt mode for the user
+    if cli_auto_prompt_env() or cli_auto_prompt:
+        # do not show auto prompt if customer is trying to access a service that they have not subscribed
+        if ctx.invoked_subcommand and not cli_util.is_service_accessible(ctx, ctx.invoked_subcommand):
+            ctx.exit()
+        cli_interactive.start_interactive_shell(ctx)
+        ctx.exit()
+
+    if ctx.obj['debug']:
+        import platform
+        click.echo(platform.platform())
+        click.echo("System name: {}".format(platform.system()))
+        click.echo("System release : {}".format(platform.release()))
+        click.echo("System version: {}\n".format(platform.version()))
+        for env in os.environ:
+            if env in ['http_proxy', 'HTTP_PROXY', 'https_proxy', 'HTTPS_PROXY', 'no_proxy', 'NO_PROXY', 'REQUESTS_CA_BUNDLE'] or 'OCI_' in env:
+                print("env {} is set".format(env))
+
+
+def cli_auto_prompt_env():
+    if cli_constants.OCI_CLI_AUTO_PROMPT_ENV_VAR in os.environ:
+        os.environ.pop(cli_constants.OCI_CLI_AUTO_PROMPT_ENV_VAR)
+        return True
+    return False
+
 
 def is_top_level_help(ctx):
     if len(sys.argv) != 3:
@@ -425,11 +570,72 @@ def is_top_level_help(ctx):
     return False
 
 
-def load_default_values(ctx, defaults_file, profile):
-    file_location = os.path.expandvars(os.path.expanduser(defaults_file))
+def load_default_values(ctx, cli_rc_file, profile):
+    file_location = os.path.expandvars(os.path.expanduser(cli_rc_file))
     ctx.obj['default_values_from_file'] = {}
     if os.path.exists(file_location):
         parser = configparser.ConfigParser(interpolation=None)
         parser.read(file_location)
         if profile in parser:
             ctx.obj['default_values_from_file'] = dict(parser.items(profile))
+
+
+def generate_oci_help(ctx):
+    if alloy_util.get_service_config_path(ctx) is None:
+        return click.wrap_text(OCI_USER_HELP, initial_indent='  ', subsequent_indent='  ', preserve_paragraphs=True).lstrip()
+    return click.wrap_text(OCI_ALLOY_USER_HELP, initial_indent='  ', subsequent_indent='  ', preserve_paragraphs=True).lstrip()
+
+
+# Need to remove help for commands under root to re-add with all the service commands
+# in order to keep formatting consistent.
+def echo_help(ctx):
+    help_text = ctx.get_help()
+    help_text = re.split('(Commands:\n)', help_text, maxsplit=1)
+    help_text_commands = help_text[2].split("\n")
+    help_text[0] = help_text[0].replace(OCI_HELP, generate_oci_help(ctx), 1)
+    help_text = help_text[0] + help_text[1]
+    command_list = []
+    # populate a dict of group and corresponding commands as key & value.
+    group_dict = {}
+    for command in help_text_commands:
+
+        # If the service is present in servicemapping.py it should not be added in non service group
+        if len(command) > 0 and command.strip().split(None, 1)[0] in service_mapping:
+            continue
+        else:
+            command_list.append(tuple(command.strip().split(None, 1)))
+    # holds value of group name currently being processed. Initialise with empty string.
+    current_group = ""
+
+    # use the imported service_mapping if this is an oci customer
+    # find the service_mapping.json if this is an Alloy customer
+    service_map = service_mapping
+
+    if alloy_util.get_service_config_path(ctx) is not None:
+        service_map = alloy_util.read_subscribed_services(ctx)
+
+    # fetch entries from service_map{} sorted by group name
+    for (service_key, service_value) in sorted(service_map.items(), key=lambda item: item[1][2]):
+        if current_group != service_value[2]:
+            if command_list:
+
+                # encountered new group, so dump the so far command_list and re-init it for new group
+                group_dict.update({current_group: command_list})
+                # empty command_list
+                command_list = []
+            current_group = service_value[2]
+
+        # update command_list with tuple of command name and desc, fetched from service_map
+        command_list.append((service_key, service_map[service_key][1]))
+
+    # Update group_dict for the last entry of above for-loop
+    group_dict.update({current_group: command_list})
+    click.echo(help_text, color=ctx.color)
+    for group_name in group_dict.keys():
+        formatter = click.formatting.HelpFormatter()
+        formatter.write(group_name)
+        formatter.write("\n\n")
+        formatter.indent()
+        with formatter.indentation():
+            formatter.write_dl(group_dict[group_name])
+        click.echo(formatter.getvalue(), color=ctx.color)
