@@ -4,25 +4,30 @@
 
 from __future__ import print_function
 import click
-from .cli_root import cli
-from .cli_constants import CLI_RC_CANNED_QUERIES_SECTION_NAME, CLI_RC_COMMAND_ALIASES_SECTION_NAME, CLI_RC_PARAM_ALIASES_SECTION_NAME, CLI_RC_DEFAULT_LOCATION
-from . import cli_util
-from .util import pymd5
+from oci_cli.cli_root import cli
+from oci_cli.cli_constants import CLI_RC_CANNED_QUERIES_SECTION_NAME, CLI_RC_COMMAND_ALIASES_SECTION_NAME, CLI_RC_PARAM_ALIASES_SECTION_NAME, CLI_RC_DEFAULT_LOCATION, OCI_CLI_AUTH_API_KEY, OCI_CLI_AUTH_SESSION_TOKEN, OCI_CLI_AUTH_INSTANCE_PRINCIPAL, OCI_CLI_AUTH_RESOURCE_PRINCIPAL
+from oci_cli import cli_util
+from services.identity.src.oci_cli_identity.generated import identity_cli
+from oci_cli.util import pymd5
+# from alloy import alloy_util
 
 import base64
 
 import configparser
 import os
 import os.path
-import stat
 import subprocess
 import sys
 import errno
+import json
+import re
+import glob
 
 from oci.regions import is_region
 from oci.regions import REGIONS
-from oci import config
+from oci import config, exceptions, signer
 
+import platform
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.backends import default_backend
 
@@ -54,6 +59,71 @@ generate_oci_config_instructions = """
 
 """
 
+instance_principal_setup_instructions = """
+    This command provides a walkthrough of setting up instance principal authentication for an OCI compute instance.
+
+    The following links provide information about topics and resources referenced by this script:
+
+    Resource OCIDs:
+
+
+    \b
+    https://docs.oracle.com/en-us/iaas/Content/General/Concepts/identifiers.htm
+
+
+    Compute Instances:
+
+
+    \b
+    https://docs.oracle.com/en-us/iaas/Content/Compute/Concepts/computeoverview.htm
+
+
+    Dynamic Groups:
+
+
+    \b
+    https://docs.oracle.com/en-us/iaas/Content/Identity/Tasks/managingdynamicgroups.htm
+
+
+    Policies:
+
+
+    \b
+    https://docs.oracle.com/en-us/iaas/Content/Identity/Concepts/policies.htm
+
+
+    In order to successfully run this command, you must:
+
+
+        - have an existing instance from which you want to run commands using instance principal authentication
+
+        - be an administrator in your tenancy so that you can create/update the necessary dynamic group and policy used to enable instance principal authentication for your instance
+
+
+    This script also requires an existing form of valid authentication to successfully run. If you do not have an existing config file and you ran this command using {api_key_auth} or {security_token_auth} auth, you will be prompted to create a new config file to use for authentication during this instance principal setup process.
+
+    \n
+
+""".format(api_key_auth=OCI_CLI_AUTH_API_KEY, security_token_auth=OCI_CLI_AUTH_SESSION_TOKEN)
+
+instance_principal_setup_end_message = """
+    \n
+    If you haven't already, you can install the CLI on your instance by connecting to your instance using SSH and running the following command:
+
+
+    \b
+    bash -c "$(curl -L https://raw.githubusercontent.com/oracle/oci-cli/master/scripts/install/install.sh)"
+
+
+    After you have installed the CLI on your instance, you can test out your instance principal authentication by connecting to your instance using SSH and running the following example command (Note: if you entered a custom policy that does not allow your dynamic group to view all compute instances in your instance's compartment, then the following example command will not work):
+
+
+    \b
+    oci compute instance list --compartment-id {compartment_id} --auth {auth}
+
+    \n
+"""
+
 upload_public_key_instructions = """
     \n
     If you haven't already uploaded your API Signing public key through the console, follow the instructions on the page linked below in the section 'How to upload the public key':
@@ -124,6 +194,7 @@ default_param_aliases = """
 """
 
 
+NO_PASSPHRASE = 'N/A'
 PUBLIC_KEY_FILENAME_SUFFIX = '_public.pem'
 PRIVATE_KEY_FILENAME_SUFFIX = '.pem'
 
@@ -134,6 +205,10 @@ DEFAULT_KEY_NAME = 'oci_api_key'
 DEFAULT_PROFILE_NAME = 'DEFAULT'
 DEFAULT_TOKEN_DIRECTORY = os.path.join(DEFAULT_DIRECTORY, 'sessions')
 DEFAULT_CONFIG_LOCATION = os.path.join(DEFAULT_DIRECTORY, 'config')
+USER_BASH_RC = os.path.expanduser(os.path.join('~', '.bashrc'))
+USER_BASH_PROFILE = os.path.expanduser(os.path.join('~', '.bash_profile'))
+DEFAULT_INSTALL_DIR = os.path.expanduser(os.path.join('~', 'lib', 'oracle-cli'))
+OCI_EXECUTABLE_NAME = 'oci.exe' if sys.platform == 'win32' else 'oci'
 
 
 @cli.group('setup', help="""Setup commands for CLI""")
@@ -158,13 +233,13 @@ def generate_key_pair(key_name, output_dir, passphrase, passphrase_file, overwri
         if not passphrase or passphrase.isspace():
             raise click.UsageError("passphrase-file must specify a password in the first line of the file")
 
-    # if no passphrase arguments are specified, then prompt for one (can still be left empty explicitly by user)
+    # if no passphrase arguments are specified, then prompt for one (can still request not to have one by user)
     if not passphrase:
-        passphrase = click.prompt(text='Enter a passphrase for your private key (empty for no passphrase)', default='', hide_input=True, show_default=False, confirmation_prompt=True)
+        passphrase = prompt_for_passphrase()
 
     output_dir = os.path.expanduser(output_dir)
     if not os.path.exists(output_dir):
-        create_directory(output_dir)
+        cli_util.create_directory(output_dir)
 
     private_key = cli_util.generate_key()
     public_key = private_key.public_key()
@@ -195,7 +270,7 @@ def generate_oci_config():
     if click.confirm("Do you want to generate a new API Signing RSA key pair? (If you decline you will be asked to supply the path to an existing key.)", default=True):
         key_location = os.path.abspath(os.path.expanduser(click.prompt(text='Enter a directory for your keys to be created', default=DEFAULT_DIRECTORY)))
         if not os.path.exists(key_location):
-            create_directory(key_location)
+            cli_util.create_directory(key_location)
 
         private_key = cli_util.generate_key()
         public_key = private_key.public_key()
@@ -205,7 +280,7 @@ def generate_oci_config():
             click.echo(config_generation_canceled_message)
             return
 
-        key_passphrase = click.prompt(text='Enter a passphrase for your private key (empty for no passphrase)', default='', hide_input=True, show_default=False, confirmation_prompt=True)
+        key_passphrase = prompt_for_passphrase()
         private_key_file = os.path.join(key_location, key_name + PRIVATE_KEY_FILENAME_SUFFIX)
         if not write_private_key_to_file(private_key_file, private_key, key_passphrase):
             click.echo(config_generation_canceled_message)
@@ -224,7 +299,7 @@ def generate_oci_config():
         fingerprint = public_key_to_fingerprint(private_key.public_key())
         click.echo("Fingerprint: {}".format(fingerprint))
 
-    if key_passphrase and not click.confirm('Do you want to write your passphrase to the config file? (if not, you will need to supply it as an argument to the CLI)', default=False):
+    if key_passphrase and not click.confirm('Do you want to write your passphrase to the config file? (If not, you will need to enter it when prompted each time you run an oci command)', default=False):
         key_passphrase = None
 
     write_config(
@@ -242,6 +317,101 @@ def generate_oci_config():
     click.echo(click.wrap_text(upload_public_key_instructions, preserve_paragraphs=True))
 
 
+@setup_group.command('instance-principal', help="""Interactive script to set up instance principal authentication for an existing compute instance.""")
+@cli_util.help_option
+@click.pass_context
+@cli_util.wrap_exceptions
+def setup_instance_principal(ctx):
+    click.echo(click.wrap_text(text=instance_principal_setup_instructions, preserve_paragraphs=True))
+
+    # get the private key passphrase (applicable for api key pair and session token auth), and if the passphrase is not None, then return it each time the passphrase is requested during this script
+    passphrase = None
+    if ctx.obj['auth'] == OCI_CLI_AUTH_API_KEY or ctx.obj['auth'] == OCI_CLI_AUTH_SESSION_TOKEN:
+        passphrase = get_passphrase(ctx)
+    if passphrase:
+        def passphrase_provider():
+            return passphrase
+        cli_util.prompt_for_passphrase = passphrase_provider
+
+    # build necessary client objects; these functions also handle authentication
+    compute_client = cli_util.build_client('core', 'compute', ctx)
+    identity_client = cli_util.build_client('identity', 'identity', ctx)
+
+    # build the config object (applicable for auth types other than instance principal or resource principal)
+    config_obj = None
+    if ctx.obj['auth'] != OCI_CLI_AUTH_INSTANCE_PRINCIPAL and ctx.obj['auth'] != OCI_CLI_AUTH_RESOURCE_PRINCIPAL:
+        config_obj = cli_util.build_config(ctx.obj)
+
+    if not click.confirm('Do you have an existing compute instance for which you would like to set up instance principal authentication?', default=True):
+        click.echo('Please create a compute instance and then run this command again. For information regarding how to create a compute instance, please visit https://docs.oracle.com/en-us/iaas/Content/GSG/Reference/overviewworkflow.htm')
+        sys.exit(1)
+
+    # get the instance OCID and the name of the compartment that the instance is in
+    instance_ocid, result = get_resource('instance', compute_client.get_instance)
+    instance_compartment_ocid = result.data.compartment_id
+    result = identity_client.get_compartment(instance_compartment_ocid)
+    instance_compartment_name = result.data.name
+
+    if click.confirm('Do you want to add this instance to an existing dynamic group?', default=False):
+        # get dynamic group OCID and update its matching rules
+        group_ocid, result = get_resource('dynamic group', identity_client.get_dynamic_group)
+        group_name = result.data.name
+        click.echo('For information about dynamic group matching rule syntax, please visit https://docs.oracle.com/en-us/iaas/Content/Identity/Tasks/managingdynamicgroups.htm#Writing')
+        current_rule = result.data.matching_rule
+        if instance_ocid in current_rule:
+            default_group_rule = current_rule
+        elif '{' in current_rule:
+            default_group_rule = "Any " + current_rule[current_rule.index('{'):current_rule.rindex('}')] + ", instance.id = '{}'}}".format(instance_ocid)
+        else:
+            default_group_rule = "Any {" + current_rule + ", instance.id = '{}'}}".format(instance_ocid)
+        group_rule = click.prompt('Enter the matching rule(s) you would like to set for this dynamic group', default=default_group_rule)
+        ctx.invoke(identity_cli.update_dynamic_group, wait_for_state='ACTIVE', dynamic_group_id=group_ocid, matching_rule=group_rule)
+
+        # check if the user wants to create/update a policy for this group
+        if click.confirm('Do you want to create a new policy for this group?', default=False):
+            new_policy = True
+        else:
+            new_policy = False
+
+        if (not new_policy) and click.confirm('Do you want to update a policy for this group?', default=False):
+            policy_ocid, result = get_resource('policy', identity_client.get_policy)
+            click.echo('For information about policy syntax, please visit https://docs.oracle.com/en-us/iaas/Content/Identity/Concepts/policysyntax.htm')
+            default_policy_statement = json.dumps(result.data.statements)
+            policy_statement = click.prompt('Enter the statement(s) you would like to set for this policy (please enter your statements in JSON format, as a bracket-enclosed list of strings)', default=default_policy_statement, value_proc=lambda statements: validate_statements(statements))
+            ctx.invoke(identity_cli.update_policy, wait_for_state='ACTIVE', policy_id=policy_ocid, statements=policy_statement)
+    else:
+        # create dynamic group and add this instance to the group using a matching rule
+        if config_obj:
+            tenancy_ocid = config_obj['tenancy']
+        else:
+            tenancy_ocid, result = get_resource('tenancy', identity_client.get_tenancy)
+        group_name = click.prompt('Enter a name for your new dynamic group', value_proc=lambda name: validate_resource_name(name))
+        group_description = click.prompt('Enter a description for your new dynamic group')
+        click.echo('For information about dynamic group matching rule syntax, please visit https://docs.oracle.com/en-us/iaas/Content/Identity/Tasks/managingdynamicgroups.htm#Writing')
+        default_group_rule = "Any {{instance.id = '{}'}}".format(instance_ocid)
+        group_rule = click.prompt('Enter the matching rule(s) you would like to set for this dynamic group', default=default_group_rule)
+        ctx.invoke(identity_cli.create_dynamic_group, wait_for_state='ACTIVE', compartment_id=tenancy_ocid, name=group_name, matching_rule=group_rule, description=group_description)
+
+        # since this is a new group that won't have an existing policy, have the user go through policy creation flow
+        new_policy = True
+
+    if new_policy:
+        # create policy for the new dynamic group
+        policy_name = click.prompt('Enter a name for the new policy to be used for your new dynamic group', value_proc=lambda name: validate_resource_name(name))
+        policy_description = click.prompt('Enter a description for the new policy to be used for your new dynamic group')
+        click.echo('WARNING: Using the default policy at the prompt below will grant all permissions for all your OCI resources to every instance in the dynamic group. Please ensure that this is the policy you want to use. If not, you can enter your custom policy statement(s) at the prompt.')
+        click.echo('For information about policy syntax, please visit https://docs.oracle.com/en-us/iaas/Content/Identity/Concepts/policysyntax.htm')
+        default_policy_statement = '["Allow dynamic-group {} to manage all-resources in compartment {}"]'.format(group_name, instance_compartment_name)
+        policy_statement = click.prompt('Enter the statement(s) you would like to set for this policy (please enter your statements in JSON format, as a bracket-enclosed list of strings)', default=default_policy_statement, value_proc=lambda statements: validate_statements(statements))
+        policy_compartment_ocid = instance_compartment_ocid
+        if click.confirm('This policy is currently set to be put in compartment {}, which is the compartment that your instance is in. Do you want to create this policy in a different compartment?'.format(instance_compartment_name), default=False):
+            policy_compartment_ocid, result = get_resource('compartment', identity_client.get_compartment)
+        ctx.invoke(identity_cli.create_policy, wait_for_state='ACTIVE', compartment_id=policy_compartment_ocid, name=policy_name, statements=policy_statement, description=policy_description)
+
+    click.echo('Successfully set up instance principal authentication for your instance!')
+    click.echo(click.wrap_text(text=instance_principal_setup_end_message.format(compartment_id=instance_compartment_ocid, auth=OCI_CLI_AUTH_INSTANCE_PRINCIPAL), preserve_paragraphs=True))
+
+
 @setup_group.command('oci-cli-rc', help="""Generates a oci_cli_rc file that can contain parameter default values and other configuration information such as command aliases and predefined queries.
 
 This command will populate the file with some default aliases and predefined queries.
@@ -333,18 +503,82 @@ def setup_autocomplete_windows():
         return
 
 
+def _get_default_rc_file():
+    bashrc_exists = os.path.isfile(USER_BASH_RC)
+    bash_profile_exists = os.path.isfile(USER_BASH_PROFILE)
+    if not bashrc_exists and bash_profile_exists:
+        return USER_BASH_PROFILE
+    if bashrc_exists and bash_profile_exists and platform.system().lower() == 'darwin':
+        return USER_BASH_PROFILE
+    return USER_BASH_RC if bashrc_exists else None
+
+
+class CLIInstallError(Exception):
+    pass
+
+
+def prompt_y_n(msg, default=None):
+    if default not in [None, 'y', 'n']:
+        raise ValueError("Valid values for default are 'y', 'n' or None")
+    y = 'Y' if default == 'y' else 'y'
+    n = 'N' if default == 'n' else 'n'
+
+    while True:
+        ans = prompt_input('{} ({}/{}): '.format(msg, y, n))
+        if ans.lower() == n.lower():
+            return False
+        if ans.lower() == y.lower():
+            return True
+        if default and not ans:
+            return default == y.lower()
+
+
+def _default_rc_file_creation_step():
+    rcfile = USER_BASH_PROFILE if platform.system().lower() == 'darwin' else USER_BASH_RC
+    ans_yes = prompt_y_n('Could not automatically find a suitable file to use. Create {} now?'.format(rcfile), default='y')
+    if ans_yes:
+        open(rcfile, 'a').close()
+        return rcfile
+    return None
+
+
+def get_rc_file_path():
+    rc_file = None
+    default_rc_file = _get_default_rc_file()
+    if not default_rc_file:
+        rc_file = _default_rc_file_creation_step()
+    rc_file = rc_file or prompt_input_with_default('Enter a path to an rc file to update (file will be created if it does not exist)', default_rc_file)
+    if rc_file:
+        rc_file_path = os.path.realpath(os.path.expanduser(rc_file))
+        if not os.path.isfile(rc_file_path):
+            print_status("Automatically created rc file at '{}'".format(rc_file_path))
+        return rc_file_path
+    return None
+
+
+def print_status(msg=''):
+    print('-- ' + msg)
+
+
+def prompt_input(msg):
+    if sys.version_info >= (3, 0):
+        return input('\n===> ' + msg)
+    else:
+        return raw_input('\n===> ' + msg)  # noqa: F821
+
+
+def prompt_input_with_default(msg, default):
+    if default:
+        return prompt_input("{} (leave blank to use '{}'): ".format(msg, default)) or default
+    else:
+        return prompt_input('{}: '.format(msg))
+
+
 def setup_autocomplete_non_windows():
-    # guidance from click is to add to .bashrc so if user has a .bashrc file, use that
-    # if .bashrc does not exist but .bash_profile does exist, we fall back to that
-    # if neither exist, we must create one. On Linux we use bashrc as recommended by click,
-    # but Mac terminal does not source .bashrc by default so use .bash_profile instead
-    bash_profile_location = os.path.expanduser('~/.bash_profile')
-    bash_rc_location = os.path.expanduser('~/.bashrc')
-    bash_config_location = bash_profile_location if sys.platform == 'darwin' else bash_rc_location
-    if os.path.exists(bash_rc_location):
-        bash_config_location = bash_rc_location
-    elif os.path.exists(bash_profile_location):
-        bash_config_location = bash_profile_location
+    click.echo("To set up autocomplete, we would update few lines in rc/bash_profile file.")
+    rc_file_path = get_rc_file_path()
+    if not rc_file_path:
+        raise CLIInstallError('No suitable profile file found.')
 
     # source bash completion script in CLI install directory
     script_relative_path = os.path.join('bin', 'oci_autocomplete.sh')
@@ -355,23 +589,40 @@ def setup_autocomplete_non_windows():
         sys.exit(1)
 
     click.echo("Using tab completion script at: {}".format(completion_script_file))
-
-    bash_profile_line = '[[ -e "{completion_script_file}" ]] && source "{completion_script_file}"'.format(completion_script_file=completion_script_file)
-    confirm_prompt = 'To set up autocomplete, we need to add a few lines to your {bash_config_location} file. Please confirm this is ok.'.format(bash_config_location=bash_config_location)
+    soft_link = os.path.expanduser("~/lib/oci_autocomplete.sh")
+    soft_link_completion_script_file = soft_link
+    bash_profile_line = '[[ -e "{soft_link_completion_script_file}" ]] && source "{soft_link_completion_script_file}"'.format(soft_link_completion_script_file=soft_link_completion_script_file)
+    confirm_prompt = 'We need to add a few lines to your {rc_file_path} file. Please confirm this is ok.'.format(rc_file_path=rc_file_path)
 
     if click.confirm(confirm_prompt, default=True):
-        with open(bash_config_location, mode='a+') as f:
+
+        if os.path.exists(soft_link):
+            os.chmod(soft_link, 0o0644)
+            os.remove(soft_link)
+        try:
+            src = completion_script_file
+            dst = soft_link
+
+            if not os.path.isdir(os.path.dirname(dst)):
+                os.makedirs(os.path.dirname(dst))
+            os.symlink(src, dst)
+        except Exception:
+            click.echo(
+                "Unable to symlink ~/lib/oci_autocomplete.sh to {}. \nAdd '{}' to your rc file and restart your terminal for the changes to take effect.".format(
+                    bash_profile_line, bash_profile_line))
+            return
+
+        with open(rc_file_path, mode='a+') as f:
             f.seek(0)
             content = f.read()
 
             # this check is not foolproof but we want to avoid adding a bunch of lines to .bash_profile/.bashrc if the user re-runs the command
-            if script_relative_path in content:
-                click.echo("It looks like tab completion for oci is already configured in {bash_config_location}. If you want to re-run the setup command please remove the line containing '{script_relative_path}' from {bash_config_location}.".format(script_relative_path=script_relative_path, bash_config_location=bash_config_location))
-                return
+            if "lib/oci_autocomplete.sh" not in content:
+                f.write('\n{}\n'.format(bash_profile_line))
+            else:
+                click.echo("It looks like tab completion for oci is already configured in {rc_file_path}. If you want to re-run the setup command please remove the line containing 'oci_autocomplete.sh' from {rc_file_path} and run oci setup authenticate again.\n".format(rc_file_path=rc_file_path))
 
-            f.write('\n{}\n'.format(bash_profile_line))
-
-        click.echo("Success!\nRun '{}' or restart your terminal for the changes to take effect.".format(bash_profile_line))
+        click.echo("Success! \n Restart your terminal or Run '{}' for the changes to take effect.".format(bash_profile_line))
     else:
         click.echo('Exiting script. Tab completion not set up.')
         return
@@ -389,7 +640,7 @@ def repair_file_permissions(file):
     if not os.path.isfile(file):
         raise click.UsageError('This command is only supported for files.')
 
-    apply_user_only_access_permissions(file)
+    cli_util.apply_user_only_access_permissions(file)
 
 
 def public_key_to_fingerprint(public_key):
@@ -429,7 +680,7 @@ def write_config(filename, user_id=None, fingerprint=None, key_file=None, tenanc
             f.write("security_token_file={}\n".format(security_token_file))
 
     # only user has R/W permissions to the config file
-    apply_user_only_access_permissions(filename)
+    cli_util.apply_user_only_access_permissions(filename)
 
 
 def write_public_key_to_file(filename, public_key, overwrite=False, silent=False):
@@ -440,7 +691,7 @@ def write_public_key_to_file(filename, public_key, overwrite=False, silent=False
         f.write(cli_util.serialize_key(public_key=public_key))
 
     # only user has R/W permissions to the key file
-    apply_user_only_access_permissions(filename)
+    cli_util.apply_user_only_access_permissions(filename)
 
     if not silent:
         click.echo('Public key written to: {}'.format(filename))
@@ -455,58 +706,13 @@ def write_private_key_to_file(filename, private_key, passphrase, overwrite=False
         f.write(cli_util.serialize_key(private_key=private_key, passphrase=passphrase))
 
     # only user has R/W permissions to the key file
-    apply_user_only_access_permissions(filename)
+    cli_util.apply_user_only_access_permissions(filename)
 
     if not silent:
         click.echo('Private key written to: {}'.format(filename))
     return True
 
 
-def apply_user_only_access_permissions(path):
-    if not os.path.exists(path):
-        raise RuntimeError("Failed attempting to set permissions on path that does not exist: {}".format(path))
-
-    if cli_util.is_windows():
-        # General permissions strategy is:
-        #   - if we create a new folder (e.g. C:\Users\opc\.oci), set access to allow full control for current user and no access for anyone else
-        #   - if we create a new file, set access to allow full control for current user and no access for anyone else
-        #   - thus if the user elects to place a new file (config or key) in an existing directory, we will not change the
-        #     permissions of that directory but will explicitly set the permissions on that file
-        username = os.environ['USERNAME']
-        userdomain = os.environ['UserDomain']
-        userWithDomain = os.environ['USERNAME']
-        if userdomain:
-            userWithDomain = userdomain + "\\" + username
-        admin_grp = '*S-1-5-32-544'
-        system_usr = '*S-1-5-18'
-        try:
-            if os.path.isfile(path):
-                subprocess.check_output('icacls "{path}" /reset'.format(path=path), stderr=subprocess.STDOUT)
-                try:
-                    subprocess.check_output('icacls "{path}" /inheritance:r /grant:r "{username}:F" /grant {admin_grp}:F /grant {system_usr}:F'.format(path=path, username=userWithDomain, admin_grp=admin_grp, system_usr=system_usr), stderr=subprocess.STDOUT)
-                except subprocess.CalledProcessError:
-                    subprocess.check_output('icacls "{path}" /inheritance:r /grant:r "{username}:F" /grant {admin_grp}:F /grant {system_usr}:F'.format(path=path, username=username, admin_grp=admin_grp, system_usr=system_usr), stderr=subprocess.STDOUT)
-            else:
-                if os.listdir(path):
-                    # safety check to make sure we aren't changing permissions of existing files
-                    raise RuntimeError("Failed attempting to set permissions on existing folder that is not empty.")
-                subprocess.check_output('icacls "{path}" /reset'.format(path=path), stderr=subprocess.STDOUT)
-                try:
-                    subprocess.check_output('icacls "{path}" /inheritance:r /grant:r "{username}:(OI)(CI)F"  /grant:r {admin_grp}:(OI)(CI)F /grant:r {system_usr}:(OI)(CI)F'.format(path=path, username=userWithDomain, admin_grp=admin_grp, system_usr=system_usr), stderr=subprocess.STDOUT)
-                except subprocess.CalledProcessError:
-                    subprocess.check_output('icacls "{path}" /inheritance:r /grant:r "{username}:(OI)(CI)F"  /grant:r {admin_grp}:(OI)(CI)F /grant:r {system_usr}:(OI)(CI)F'.format(path=path, username=username, admin_grp=admin_grp, system_usr=system_usr), stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as exc_info:
-            print("Error occurred while attempting to set permissions for {path}: {exception}".format(path=path, exception=str(exc_info)))
-            sys.exit(exc_info.returncode)
-    else:
-        if os.path.isfile(path):
-            os.chmod(path, stat.S_IRUSR | stat.S_IWUSR)
-        else:
-            # For directories, we need to apply S_IXUSER otherwise it looks like on Linux/Unix/macOS if we create the directory then
-            # it won't behave like a directory and let files be put into it
-            os.chmod(path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
-
-
 def validate_private_key_passphrase(filename, passphrase):
     with open(filename, 'rb') as f:
         try:
@@ -567,9 +773,31 @@ def validate_ocid(ocid, pattern):
     return ocid
 
 
-def create_directory(dirname):
-    os.makedirs(dirname)
-    apply_user_only_access_permissions(dirname)
+def validate_resource_ocid(ocid):
+    # the link provided in the error message in the validate_ocid function is not applicable for general resource OCIDs
+    # so, the validate_ocid function should not be used to validate general resource OCIDs since the link in the error message could be misleading
+    pattern = re.compile("^(ocid1.)([0-9a-zA-Z-_]+.)(oc[1-3].)([0-9a-zA-Z-_]*.)([0-9a-zA-Z-_]+)$")
+    if not pattern.match(ocid):
+        raise click.BadParameter("Invalid OCID format. OCID syntax can be found here: https://docs.oracle.com/en-us/iaas/Content/General/Concepts/identifiers.htm#Oracle")
+
+    return ocid
+
+
+def validate_statements(statements):
+    try:
+        json.loads(statements)
+    except ValueError:
+        raise click.BadParameter("Please enter your statements in JSON format, as a bracket-enclosed list of strings")
+
+    return statements
+
+
+def validate_resource_name(name):
+    pattern = re.compile("^[a-zA-Z0-9._-]+$")
+    if not pattern.match(name):
+        raise click.BadParameter("Invalid resource name. The name cannot contain spaces; only letters, numerals, hyphens, periods, or underscores are allowed.")
+
+    return name
 
 
 def validate_profile_name(value, config_parser, overwrite=False, makeUpper=True):
@@ -619,7 +847,7 @@ def prompt_for_config_location():
 
         # if user inputs only a filename (dirname=='') it implies the current directory so no need to create a dir
         if dirname and not os.path.exists(dirname):
-            create_directory(dirname)
+            cli_util.create_directory(dirname)
 
     return (config_location, DEFAULT_PROFILE_NAME)
 
@@ -653,6 +881,14 @@ def prompt_for_region():
     return sorted_region_list[int(region) - 1] if is_valid_region_index(region) else region
 
 
+def prompt_for_passphrase():
+    passphrase = click.prompt(
+        text='Enter a passphrase for your private key ("{}" for no passphrase)'.format(NO_PASSPHRASE),
+        default=None, hide_input=True, show_default=False, confirmation_prompt=True
+    )
+    return None if passphrase == NO_PASSPHRASE else passphrase
+
+
 def remove_profile_from_config(config_file, profile_name_to_terminate):
     # Set default_section to custom value or else we can't delete 'DEFAULT'
     # profiles since it is protected by configparser
@@ -661,3 +897,146 @@ def remove_profile_from_config(config_file, profile_name_to_terminate):
     config.remove_section(profile_name_to_terminate)
     with open(config_file, 'w') as config_file_handle:
         config.write(config_file_handle)
+
+
+def get_resource(resource_type, get_func):
+    prompt_ocid = True
+    while prompt_ocid:
+        resource_ocid = click.prompt('Enter the {resource} OCID'.format(resource=resource_type), value_proc=lambda ocid: validate_resource_ocid(ocid))
+        try:
+            result = get_func(resource_ocid)
+            prompt_ocid = False
+        except exceptions.ServiceError as e:
+            if e.code == 'NotAuthorizedOrNotFound':
+                click.echo('Error: Could not find {resource} with the given OCID; please ensure that you have entered the correct {resource} OCID and that you are authorized to access the {resource}'.format(resource=resource_type))
+            else:
+                click.echo('Error: ' + e.message)
+                sys.exit(1)
+
+    return resource_ocid, result
+
+
+def get_passphrase(ctx):
+    try:
+        client_config = cli_util.build_config(ctx.obj)
+        private_key = signer.load_private_key_from_file(client_config['key_file'], client_config['pass_phrase'])
+    except exceptions.ConfigFileNotFound:
+        # if the user is missing a config file, they will be prompted to create one during the build_client flow, which is called later in the instance principal setup process
+        # so, we don't have to handle this exception here, and can just return None as the passphrase
+        return None
+    except exceptions.MissingPrivateKeyPassphrase:
+        # if this exception is thrown, then the client_config was built successfully, but the private key was encrypted with a passphrase that isn't in the config
+        # so, we can prompt for the private key passphrase once, and then have it be returned each time this script invokes another CLI function that needs the passphrase
+        # if the entered passphrase is incorrect, the script will terminate with the exception thrown by load_private_key_from_file
+        passphrase = cli_util.prompt_for_passphrase()
+        private_key = signer.load_private_key_from_file(client_config['key_file'], passphrase)
+        return passphrase
+    return None
+
+
+@setup_group.command('find-installations', help="""Finds and lists locations of all default OCI CLI installs.""")
+@cli_util.help_option
+@click.pass_context
+@cli_util.wrap_exceptions
+def find_cli_installations(ctx):
+
+    package_locations = []
+    executable_locations = []
+
+    # check for installer script installation
+    click.echo('Checking for CLI installer script default installation...')
+    if os.path.exists(DEFAULT_INSTALL_DIR):
+        # if the above condition is satisfied, that means the CLI package was installed at the default location from the installer script
+        installation_loc = DEFAULT_INSTALL_DIR
+        package_locations.append(('CLI Installer Script', installation_loc))
+
+    # check for homebrew installation
+    click.echo('Checking for Homebrew installation...')
+    result = subprocess.run(['brew list'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                            universal_newlines=True)
+    if (result.returncode == 0) and ('oci-cli' in result.stdout):
+        # if the above condition is satisfied, that means oci-cli is an installed homebrew package
+        # get the oci-cli installation location
+        installation_loc = []
+        result = subprocess.run(['brew --cellar oci-cli'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                universal_newlines=True)
+        installation_loc.append(result.stdout.strip())
+        result = subprocess.run(['brew --prefix oci-cli'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                universal_newlines=True)
+        installation_loc.append(result.stdout.strip())
+        package_locations.append(('Homebrew', ', '.join(installation_loc)))
+
+    # check for yum installation
+    click.echo('Checking for yum installation...')
+    result = subprocess.run(['yum list installed'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                            universal_newlines=True)
+    if (result.returncode == 0) and ('oci-cli' in result.stdout):
+        # if the above condition is satisfied, that means the CLI is an installed yum package
+        # CLI yum package name has the form pythonXX-oci-cli; the python version determines where the package is installed
+        package_name = re.findall('python[0-9]+-oci-cli', result.stdout)
+        python_version_num = re.findall('[0-9]+', package_name[0])
+        python_version = '.'.join(list(python_version_num[0]))
+        installation_loc = '/usr/lib/python{version}/site-packages/oci_cli'.format(version=python_version)
+        package_locations.append(('yum', installation_loc))
+
+    # check for pip installation
+    click.echo('Checking for pip installation...')
+    result = subprocess.run(['pip3 show oci-cli'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                            universal_newlines=True)
+    if result.returncode == 0:
+        # if the above condition is satisfied, that means oci-cli is an installed pip package
+        # parse result string to get the oci-cli installation location
+        location_line = re.findall('Location: .*\n', result.stdout)
+        installation_loc = os.path.join(location_line[0].split()[1], 'oci_cli')
+        package_locations.append(('pip3', installation_loc))
+
+    # check for oci executables on the user's PATH
+    click.echo('Searching for CLI executables on your $PATH...')
+    try:
+        if 'PATH' in os.environ:
+            path_env = os.environ['PATH']
+            click.echo('Your current $PATH is {}'.format(path_env))
+            for path in path_env.split(os.pathsep):
+                search_path = os.path.join(path, '**', OCI_EXECUTABLE_NAME)
+                executable_locations.extend(glob.glob(search_path, recursive=True))
+        else:
+            click.echo('No PATH variable found in your environment')
+    except Exception as ex:
+        pass
+    click.echo()
+
+    # print the installations that were found
+    if len(package_locations) == 0:
+        click.echo('No OCI CLI package installations were found')
+    else:
+        click.echo('The following OCI CLI package installations were found:\n')
+        for i in range(len(package_locations)):
+            click.echo('\t[{index}] {method}: {location}'.format(index=i + 1, method=package_locations[i][0],
+                                                                 location=package_locations[i][1]))
+    click.echo()
+
+    # print the installations that were found
+    if len(executable_locations) == 0:
+        click.echo('No OCI CLI executables were found on your $PATH')
+    else:
+        click.echo('The following OCI CLI executables were found on your $PATH:\n')
+        for i in range(len(executable_locations)):
+            click.echo('\t[{index}] {location}'.format(index=i + 1, location=executable_locations[i]))
+
+    click.echo()
+
+    click.echo(click.wrap_text(
+        text="Visit https://docs.oracle.com/en-us/iaas/Content/API/SDKDocs/cliuninstall.htm to uninstall OCI-CLI",
+        preserve_paragraphs=True))
+    find_install_note = 'PLEASE NOTE: This command lists CLI installations in default locations and CLI executables on your $PATH. There may be additional (unlisted) CLI installations at other non-default locations or CLI executables that are not on your current $PATH.'
+    click.echo(click.wrap_text(text=find_install_note, preserve_paragraphs=True))
+
+    ctx.exit()
+
+
+# @setup_group.command('service-config', help='Load subscribed services from service config file, which should reside in the same compartment as the config file.')
+# @cli_util.help_option
+# @click.pass_context
+# @cli_util.wrap_exceptions
+# def setup_service_config(ctx):
+#     alloy_util.write_subscribed_services(ctx)
